#!/usr/bin/ruby
# $Id: parse-oldcurlists.rb 7141 2019-12-27 18:51:08Z flaterco $
#
# The undocumented MDAPI (1.0) geogroup children query (getting stations)
# does not work for currents.  In lieu of that, parse geogroups out of the
# old style station list web pages and populate a table with them.
#
# Precedents:
#   harmbase2-20161231/nos/preparse_current_predictions_pages.rb
#   harmbase2-20161231/importNOSSC.pgcc

# Input: Stations?g=*
# 3 levels of headings
#   1. <h1><small>
#   2. <h5>
#   3. <h5>&nbsp;&nbsp;&nbsp;&nbsp;
# 2 levels of stations
#   1. N/A
#   2. <td>&nbsp;&nbsp;&nbsp;&nbsp;<a
#   3. <td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a
# "Weak and Variable" stations with no predictions lack the link to
# <a href='Predictions?id=*

require "set"
require "pg"

# Complete mapping from harmbase2-20161231/importNOSSC.pgcc hdr2_fixups
# Subsequent changes:
# Added spelling fix STRAIGHT --> Strait
# Added WASHINGTON-BRITISH COLUMBIA COAST
hdr2_fixups = {
  "NANTUCKET SOUND ENTRANCE" => "Nantucket Sound entrance",
  "NANTUCKET SOUND" => "Nantucket Sound",
  "VINEYARD SOUND" => "Vineyard Sound",
  "NARRAGANSETT BAY" => "Narragansett Bay",
  "EAST RIVER" => "East River",
  "HARLEM RIVER" => "Harlem River",
  "KILL VAN KULL" => "Kill Van Kull",
  "CHESAPEAKE BAY" => "Chesapeake Bay",
  "POTOMAC RIVER" => "Potomac River",
  "PATUXENT RIVER" => "Patuxent River",
  "LITTLE CHOPTANK RIVER" => "Little Choptank River",
  "CHOPTANK RIVER" => "Choptank River",
  "EASTERN BAY" => "Eastern Bay",
  "CHESTER RIVER" => "Chester River",
  "SASSAFRASS RIVER" => "Sassafrass River",
  "ELK RIVER" => "Elk River",
  "PAMLICO SOUND" => "Pamlico Sound",
  "CAPE FEAR RIVER" => "Cape Fear River",
  "WINYAH BAY" => "Winyah Bay",
  "CHARLESTON HARBOR" => "Charleston Harbor",
  "STONO RIVER" => "Stono River",
  "PORT ROYAL SOUND" => "Port Royal Sound",
  "CALIBOGUE SOUND" => "Calibogue Sound",
  "SAVANNAH RIVER" => "Savannah River",
  "WASSAW SOUND" => "Wassaw Sound",
  "OSSABAW SOUND" => "Ossabaw Sound",
  "ST. CATHERINES SOUND" => "St. Catherines Sound",
  "SAPELO SOUND" => "Sapelo Sound",
  "DOBOY SOUND" => "Doboy Sound",
  "ALTAMAHA SOUND" => "Altamaha Sound",
  "ST. SIMONS SOUND" => "St. Simons Sound",
  "ST. ANDREWS SOUND" => "St. Andrews Sound",
  "ST. JOHNS RIVER" => "St. Johns River",
  "SARASOTA BAY" => "Sarasota Bay",
  "TAMPA BAY" => "Tampa Bay",
  "APALACHEE BAY" => "Apalachee Bay",
  "GALVESTON BAY" => "Galveston Bay",
  "SAN DIEGO BAY" => "San Diego Bay",
  "YAQUINA BAY" => "Yaquina Bay",
  "TILLAMOOK BAY" => "Tillamook Bay",
  "GRAYS HARBOR" => "Grays Harbor",
  "STRAIGHT OF JUAN DE FUCA" => "Strait of Juan de Fuca",
  "ADMIRALTY INLET" => "Admiralty Inlet",
  "HOOD CANAL" => "Hood Canal",
  "PUGET SOUND" => "Puget Sound",
  "ROSARIO STRAIT" => "Rosario Strait",
  "SAN JUAN CHANNEL" => "San Juan Channel",
  "GEORGIA STRAIT" => "Georgia Strait",
  "DIXON ENTRANCE" => "Dixon Entrance",
  "PEARSE CANAL" => "Pearse Canal",
  "PORTLAND CANAL" => "Portland Canal",
  "REVILLAGIGEDO CHANNEL" => "Revillagigedo Channel",
  "TONGASS NARROWS" => "Tongass Narrows",
  "FELICE STRAIT" => "Felice Strait",
  "NICHOLS PASSAGE" => "Nichols Passage",
  "BEHM CANAL" => "Behm Canal",
  "CLARENCE STRAIT" => "Clarence Strait",
  "ERNEST SOUND" => "Ernest Sound",
  "ZIMOVIA STRAIT" => "Zimovia Strait",
  "CORDOVA BAY" => "Cordova Bay",
  "TLEVAK STRAIT" => "Tlevak Strait",
  "MEARES PASSAGE" => "Meares Passage",
  "ULLOA CHANNEL" => "Ulloa Channel",
  "EL CAPITAN PASSAGE" => "El Capitan Passage",
  "SUMNER STRAIT" => "Sumner Strait",
  "KEKU STRAIT" => "Keku Strait",
  "WRANGELL NARROWS" => "Wrangell Narrows",
  "FREDERICK SOUND" => "Frederick Sound",
  "STEPHENS PASSAGE" => "Stephens Passage",
  "CHATHAM STRAIT" => "Chatham Strait",
  "SITKA SOUND" => "Sitka Sound",
  "KRESTOF SOUND" => "Krestof Sound",
  "NEVA STRAIT" => "Neva Strait",
  "SALISBURY SOUND" => "Salisbury Sound",
  "PERIL STRAIT" => "Peril Strait",
  "ICY BAY" => "Icy Bay",
  "CONTROLLER BAY" => "Controller Bay",
  "SHUMAGIN ISLANDS" => "Shumagin Islands",
  "UNIMAK ISLAND" => "Unimak Island",
  "ISANOTSKI STRAIT" => "Isanotski Strait",
  "ALEUTIAN ISLANDS" => "Aleutian Islands",
  "PORT MOLLER" => "Port Moller",
  "KVICHAK BAY" => "Kvichak Bay",
  "KUSKOKWIM BAY" => "Kuskokwim Bay",

  # Redundant ones
  "PUERTO RICO" => nil,
  "MASSACHUSETTS COAST" => nil,
  "NORTH CAROLINA COAST" => nil,
  "SOUTH CAROLINA COAST" => nil,
  "TEXAS COAST" => nil,
  "OREGON COAST" => nil,
  "WASHINGTON COAST" => nil,
  "WASHINGTON-BRITISH COLUMBIA COAST" => nil,
  "ALASKA PENINSULA" => nil,
  "BERING SEA" => nil,

  # Ambiguous ones
  "SEVERN and MAGOTHY RIVERS" => nil,
  "BACK => GUNPOWDER and BUSH RIVERS" => nil,
  "NEW and WRIGHT RIVERS" => nil,
  "FLORIDA REEFS to MIDNIGHT PASS" => nil,
  "BOCA CIEGA BAY and ST. JOSEPH SOUND" => nil,
  "POSSESSION SOUND-SKAGIT BAY" => nil,
  "HARO STRAIT AND BOUNDARY PASS" => nil,
  "HECATE STRAIT AND CHATHAM SOUND" => nil,
  "BLAKE CHANNEL AND EASTERN PASSAGE" => nil,
  "BUCARELLI BAY TO DAVIDSON INLET" => nil,
  "NAKWASINA SOUND AND PASSAGE" => nil,
  "HERENDEEN BAY - PORT HEIDEN" => nil,
  "NUSHAGAK BAY AND APPROACHES" => nil,
  # Ugh...
  "COLUMBIA RIVER AND APPROACHES" => nil
}

db = PG::Connection.open(:dbname => "harmbase2")
begin
  res = db.exec("
    create table currents_geogroups (
      sid    text primary key,  -- NOT sidplus, NO bin numbers
      state  text not null,     -- level 5 in MDAPI tides
      region text,              -- level 6, sometimes nulled out
      subregion text            -- level 7, often redundant with station name
    )
  ")
rescue PG::DuplicateTable
  puts "Table currents_geogroups already exists; continuing..."
end

numlvls = 3
headers = Array.new(numlvls,"ERROR")
donesids = Set.new

lines = ARGF.readlines # slurp
i=0
while i < lines.size
  line = lines[i]
  if line.include? "<h1>"
    f = line.scan(/<h1><small>([^<]+)/)[0]
    headers[0] = f[0]
    headers.fill("ERROR",1..(numlvls-1))
    i += 1
  elsif line.include? "<h5>"
    if line.include? "&nbsp;"
      f = line.scan(/<h5>(&nbsp;)+([^<]+)/)[0]
      headers[2] = f[1]
      headers.fill("ERROR",3..(numlvls-1))
    else
      f = line.scan(/<h5>([^<]+)/)[0]
      if hdr2_fixups.include?(f[0])
        headers[1] = hdr2_fixups[f[0]]
      else
	raise "Have no fixup for geogroup name #{f[0]}"
      end
      headers.fill("ERROR",2..(numlvls-1))
    end
    i += 1
  # preparse_current_predictions_pages.rb skipped the "Weak and variable"
  # ones, but we need them now.  We don't need the sidplus from the
  # predictions link, coordinates, or other misc.
  elsif line.include? "onmouseover"
    hlvl = line.scan(/&nbsp/).size / 4
    i += 2
    line = lines[i]
    f = line.scan(/>([^<]+)/)[0]
    # There are some weird sids in here that aren't of the standard
    # 3-letter 4-digit format.
    sid = f[0]
    i += 9

    unless donesids.include?(sid)
      res = db.exec("insert into currents_geogroups values ($1, $2, $3, $4)",
        [sid, headers[0], headers[1], hlvl < 2 ? nil : headers[2]]);
      donesids.add(sid)
    end

  else
    i += 1
  end
end
